//--
//--	palette.cpp
//--
//--	TBlackIdentityPalette / TIdentityPalette Class Definition Module
//--
//     These classes are used to insure that you will get a
//      palette that has all the colors in your bitmaps.
//--

#include <owl\gdibase.h>
#include <owl\gdiobjec.h>
#include <owl\dc.h>
#pragma hdrstop
#include "palette.h"



//--------------------------------------------------
//  TBlackIdentityPalette::TBlackIdentityPalette()
//--------------------------------------------------
TBlackIdentityPalette::TBlackIdentityPalette() : TPalette((HPALETTE)1)
{
	TScreenDC ScreenDC;
	int i;
	UINT nCount;

	// We fooled TPalette::TPalette() by passing a phony handle (above) so
	//    we'll set it back to zero here.
	Handle = 0;

	// Get identity palette entry count
	if (ScreenDC.GetDeviceCaps(RASTERCAPS) & RC_PALETTE)
	{
		// Palette device...easy!
		nCount = (UINT)ScreenDC.GetDeviceCaps(SIZEPALETTE);
	}
	else
	{
		// Non-palette device.  We'll build a 256 entry color table for
		//   all devices
		nCount = 256;
	}

	// allocate and init a temporary LOGPALETTE structure
	LOGPALETTE *pLogPal = (LOGPALETTE *)new BYTE[sizeof(LOGPALETTE) +
										(nCount - 1) * sizeof(PALETTEENTRY)];
	pLogPal->palVersion  = 0x300;
	pLogPal->palNumEntries = (WORD)nCount;

	for (i = 0; i < nCount; i++)
	{
		pLogPal->palPalEntry[i].peRed = 0;
		pLogPal->palPalEntry[i].peGreen = 0;
		pLogPal->palPalEntry[i].peBlue = 0;
		pLogPal->palPalEntry[i].peFlags = PC_RESERVED;
	}

	// create the palette
	Handle = ::CreatePalette(pLogPal);
	TGdiBase::ShouldDelete = TRUE;

	delete pLogPal;

	CheckValid();
	OBJ_REF_ADD(Handle,Palette);//???
}


//----------------------------------------
//  TIdentityPalette::TIdentityPalette()
//----------------------------------------
TIdentityPalette::TIdentityPalette() : TBlackIdentityPalette()
{
	TScreenDC ScreenDC;
	int i;
	UINT nCount, nSysColorCount;

	// Get identity palette entry count and
	//    system (static) color count ("nStaticCount" member variable)
	nStaticCount = (WORD)ScreenDC.GetDeviceCaps(NUMCOLORS);
	if (ScreenDC.GetDeviceCaps(RASTERCAPS) & RC_PALETTE)
	{
		// Palette device...easy!
		nCount = (UINT)ScreenDC.GetDeviceCaps(SIZEPALETTE);
	}
	else
	{
		// Non-palette device.  Table has 256 entries for all devices.
		nCount = 256;
	}

	// allocate and init a temporary PALETTEENTRY array
	PALETTEENTRY *pPalEntries = new PALETTEENTRY[nCount];

	// fill the array with the TBlackIdentityPalette default entries
	::GetPaletteEntries((HPALETTE)*this,0,nCount,pPalEntries);

	// For SYSPAL_NOSTATIC, just replace the first and last entries with
	//    black and white
	if (ScreenDC.GetSystemPaletteUse() == SYSPAL_NOSTATIC)
	{
		nStaticCount = 2;

		pPalEntries[0].peFlags = 0;

		pPalEntries[nCount - 1].peRed = 255;
		pPalEntries[nCount - 1].peGreen = 255;
		pPalEntries[nCount - 1].peBlue = 255;
		pPalEntries[nCount - 1].peFlags = 0;
	}
	else
	{
		nSysColorCount = nStaticCount / 2;

		// Get the lower static colors from the system palette
		ScreenDC.GetSystemPaletteEntries(0,nSysColorCount,pPalEntries);

		// Set the peFlags of the lower static colors to zero
		for (i = 0; i < nSysColorCount; ++i)
		{
			pPalEntries[i].peFlags = 0; // system colors
		}

		// Get the upper static colors from the system palette
		ScreenDC.GetSystemPaletteEntries(nCount - nSysColorCount,
									  nSysColorCount,
									  &pPalEntries[nCount - nSysColorCount]);

		// Set the peFlags of the upper static colors to zero
		for (i = nCount - nSysColorCount; i < nCount; ++i)
		{
			pPalEntries[i].peFlags = 0;
		}
	}

	// place the entries back in the palette
	::SetPaletteEntries((HPALETTE)*this,0,nCount,pPalEntries);

	delete pPalEntries;
}


//------------------------------------------
//  TIdentityPalette::RemoveStaticColors()
//------------------------------------------
void TIdentityPalette::RemoveStaticColors()
{
	int i;
	UINT nSysColorCount;

	if (nStaticCount > 2)
	{

		UINT nCount = GetNumEntries();

		// allocate and init a temporary PALETTEENTRY array
		PALETTEENTRY *pPalEntries = new PALETTEENTRY[nCount];

		// fill the array with current entries
		::GetPaletteEntries((HPALETTE)*this,0,nCount,pPalEntries);

		nSysColorCount = nStaticCount / 2 - 1;

		// Set the entries of the lower static colors to black/reserved
		//   (leave black entry alone)
		for (i = 1; i < nSysColorCount; ++i)
		{
			pPalEntries[i].peRed = 0;
			pPalEntries[i].peGreen = 0;
			pPalEntries[i].peBlue = 0;
			pPalEntries[i].peFlags = PC_RESERVED;
		}

		// Set the entries of the upper static colors to black/reserved
		//   (leave white entry alone)
		for (i = nCount - (nSysColorCount + 1); i < nCount - 1; ++i)
		{
			pPalEntries[i].peRed = 0;
			pPalEntries[i].peGreen = 0;
			pPalEntries[i].peBlue = 0;
			pPalEntries[i].peFlags = PC_RESERVED;
		}

		// place the entries back in the palette
		::SetPaletteEntries((HPALETTE)*this,0,nCount,pPalEntries);

		delete pPalEntries;
	}
}



